# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import re
import asyncio
import traceback
from uuid import uuid4
from json import JSONDecodeError, dumps

from geosnipe.constants import PAYLOAD, USER_AGENT


class Mojang(object):
    @staticmethod
    async def login(email: str, password: str):
        data = dumps({
            "agent": {
                "name": "Minecraft",
                "version": 1
            },
            "username": email,
            "password": password,
            "clientToken": str(uuid4()),
            "requestUser": True
        })

        payload = PAYLOAD % (bytes(str(len(data)), "utf-8"), USER_AGENT, bytes(data, "utf-8"))

        try:
            reader, writer = await asyncio.open_connection("authserver.mojang.com", 443, ssl=True)
            writer.write(payload)
            await writer.drain()
            text = str(await reader.read(1024), "utf-8")
            writer.close()
            return re.findall("\"accessToken\":\"(.*?)\",", text)[0]
        except KeyError:
            traceback.print_exc()
            return None
        except ConnectionError:
            traceback.print_exc()
            return None
        except JSONDecodeError:
            traceback.print_exc()
            return None
        except IndexError:
            traceback.print_exc()
            return None



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import argparse


main_parser = argparse.ArgumentParser()
service_subparsers = main_parser.add_subparsers(title="GeoSnipe Commands", dest="service_command")


def create_offset_parser() -> None:
    offset_parser = argparse.ArgumentParser(add_help=False)
    offset_parser.add_argument(
        "--tests", "-t", type=int, help="Amount of times you want to test | DEFAULT: 5", default=5
    )
    service_subparsers.add_parser("offset", help="Find your offset", parents=[offset_parser])


def create_sniper_parser() -> None:
    snipe_parser = argparse.ArgumentParser(add_help=False)
    snipe_parser.add_argument("username", metavar="username", type=str, help="Username you want to snipe")
    snipe_parser.add_argument("drop_time", metavar="drop_time", type=float, help="What time the username drops")
    snipe_parser.add_argument("offset", metavar="offset", type=float, help="The offset of the snipe")
    snipe_parser.add_argument(
        "--bearer", "-b", type=str, help="The bearer token of the account | DEFAULT: None", default=None
    )
    snipe_parser.add_argument(
        "--combo", "-c", type=str, help="The EMAIL:PASSWORD of the account | DEFAULT: None", default=None
    )
    snipe_parser.add_argument(
        "--auth", "-a", type=str,
        help="Authentication you want to use (mojang or microsoft) | DEFAULT: mojang", default="mojang"
    )
    service_subparsers.add_parser("snipe", help="Snipe a username", parents=[snipe_parser])


def create_sshoffset_parser() -> None:
    sshoffset_parser = argparse.ArgumentParser(add_help=False)
    sshoffset_parser.add_argument("vps", metavar="vps", type=str, help="The VPS you want to use")
    sshoffset_parser.add_argument(
        "--tests", "-b", type=str, help="Amount of times you want to test | DEFAULT: 5", default=5
    )
    service_subparsers.add_parser("sshoffset", help="Find offset on VPS", parents=[sshoffset_parser])


def create_sshsniper_parser() -> None:
    sshsnipe_parser = argparse.ArgumentParser(add_help=False)
    sshsnipe_parser.add_argument("vps", metavar="vps", type=str, help="The VPS you want to use")
    sshsnipe_parser.add_argument("username", metavar="username", type=str, help="Username you want to snipe")
    sshsnipe_parser.add_argument("drop_time", metavar="drop_time", type=float, help="What time the username drops")
    sshsnipe_parser.add_argument("offset", metavar="offset", type=float, help="The offset of the snipe")
    sshsnipe_parser.add_argument(
        "--bearer", "-b", type=str, help="The bearer token of the account | DEFAULT: None", default=None
    )
    sshsnipe_parser.add_argument(
        "--combo", "-c", type=str, help="The EMAIL:PASSWORD of the account | DEFAULT: None", default=None
    )
    sshsnipe_parser.add_argument(
        "--auth", "-a", type=str,
        help="Authentication you want to use (mojang or microsoft) | DEFAULT: mojang", default="mojang"
    )
    service_subparsers.add_parser("sshsnipe", help="Snipe a username on a VPS", parents=[sshsnipe_parser])


def create_sshinstall_parser() -> None:
    sshinstall_parser = argparse.ArgumentParser(add_help=False)
    sshinstall_parser.add_argument("vps", metavar="vps", type=str, help="The VPS you want to use")
    service_subparsers.add_parser("sshinstall", help="Install GeoSnipe on a VPS", parents=[sshinstall_parser])


def create_version_parser() -> None:
    version_parser = argparse.ArgumentParser(add_help=False)
    service_subparsers.add_parser("version", help="Check the version of GeoSnipe", parents=[version_parser])


def parse_args() -> argparse.Namespace:
    create_offset_parser()
    create_sniper_parser()
    create_sshoffset_parser()
    create_sshsniper_parser()
    create_sshinstall_parser()
    create_version_parser()
    return main_parser.parse_args()



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import asyncio
from time import time
from random import choice
from typing import Optional
from string import ascii_letters

from paramiko.ssh_exception import SSHException

from geosnipe.ssh import Client
from geosnipe.github import GitHub
from geosnipe.auth.mojang import Mojang
from geosnipe.progressbar import ProgressBar
from geosnipe.auth.microsoft import Microsoft
from geosnipe import logger as logging, __version__
from geosnipe.constants import SKIN, MOJANG, MICROSOFT


async def check_arguments(username: str, drop_time: int,
                          offset: int, bearer: Optional[str] = None,
                          combo: Optional[str] = None, auth: str = "mojang") -> Optional[str]:
    if bearer is None and combo is None:
        return "Must have either the `bearer` or `combo` argument."
    elif auth.lower() not in ("mojang", "microsoft"):
        return "The argument `auth` must be mojang or microsoft."
    elif drop_time - (offset / 1000) < time():
        return "Username has already dropped."
    elif combo is not None:
        if combo.count(":") != 1:
            return "Combo is not valid."
    elif len(username) < 3:
        return "This username is too short."


async def login(email: str, password: str, auth: str) -> Optional[str]:
    if auth == "microsoft":
        return await Microsoft.login(email, password)
    else:
        return await Mojang.login(email, password)


async def send_request(reader: asyncio.StreamReader, writer: asyncio.StreamWriter,
                       data: bytes, delay: float) -> tuple[str, float]:
    await asyncio.sleep(delay)
    writer.write(data)
    await writer.drain()
    return str((await reader.read(12)), "utf-8")[9:12], time()


async def change_skin(access_token: str) -> str:
    reader, writer = await asyncio.open_connection("api.minecraftservices.com", 443, ssl=True)
    writer.write(SKIN % bytes(access_token, "utf-8"))
    await writer.drain()
    return str((await reader.read(12)), "utf-8")[9:12]


async def create_payload(username: str, bearer: str, auth: str = "mojang") -> bytes:
    if auth == "microsoft":
        content = "{\"profileName\": \"%s\"}" % username
        return MICROSOFT % (bytes(bearer, "utf-8"), bytes(str(len(content)), "utf-8"), bytes(content, "utf-8"))
    else:
        return MOJANG % (bytes(username, "utf-8"), bytes(bearer, "utf-8"))


async def snipe(username: str, drop_time: int,
                offset: int, bearer: Optional[str] = None,
                combo: Optional[str] = None, auth: str = "mojang") -> Optional[tuple]:
    args_check = await check_arguments(username, drop_time, offset, bearer, combo, auth)

    if args_check is not None:
        logging.error(args_check)
        return None

    if bearer is None:
        logging.info(f"Authenticating using `{auth}`.")

        spl = combo.split(":")
        bearer = await login(spl[0], spl[1], auth)

        if bearer is None:
            logging.info(f"Failed to authenticate using `{auth}`.")
            return None

    payload = await create_payload(username, bearer, auth)
    del spl, auth, combo

    logging.info(f"Sniping `{username}` at {drop_time - offset / 1000}.")

    await asyncio.sleep(drop_time - time() - 3)

    try:
        logging.info("Opening connections...")
        connections = await asyncio.gather(
            *[asyncio.open_connection("api.minecraftservices.com", 443, ssl=True) for _ in range(3)]
        )
    except ConnectionError:
        logging.error("Failed to open connections.")
        return

    await asyncio.sleep(drop_time - time() - offset / 1000)

    try:
        results = await asyncio.gather(
            *[send_request(connections[i][0], connections[i][1], payload, 0.001 * i) for i in range(len(connections))]
        )
    except ConnectionError:
        logging.error("Failed to change username.")
        return

    try:
        for conn in connections:
            conn[1].close()
    except ConnectionError:
        logging.error("Failed to close socket.")

    for result in results:
        logging.info(f"Received {result[0]} at {result[1]}")

        if result[0] == "200":
            logging.info("Changing skin...")

            try:
                await change_skin(bearer)
            except ConnectionError:
                logging.error("Failed to change skin.")

    return results


async def offset(tests: int = 5) -> Optional[float]:
    if tests < 1:
        logging.error("The argument `tests` cannot be less than or equal to 0.")
        return None

    data = bytes(
        f"PUT /minecraft/profile/name/{''.join([choice(ascii_letters) for _ in range(5)])} HTTP/1.1\r\n"
        f"Host: api.minecraftservices.com\r\n"
        f"Content-Type: application/json\r\n"
        f"Authorization: Bearer {''.join([choice(ascii_letters) for _ in range(250)])}\r\n\r\n",
        "utf-8"
    )
    times = []

    logging.info("Finding offset...")

    progress_bar = ProgressBar(tests)

    try:
        for _ in range(tests):
            conn = await asyncio.open_connection("api.minecraftservices.com", 443, ssl=True)
            start = time()
            conn[1].write(data)
            await conn[1].drain()
            await conn[0].read(12)
            end = time()
            times.append(end - start)
            progress_bar.update()
    except ConnectionError:
        logging.error("Failed to find offset.")
        return

    logging.info(f"Your offset is {round(sum(times) / 10 * 1000, 2)} ms.")


async def get_version():
    logging.info(f"You are using GeoSnipe {__version__}")


async def sshsnipe(vps: str, username: str, drop_time: int,
                    offset: int, bearer: Optional[str] = None,
                    combo: Optional[str] = None, auth: str = "mojang") -> None:
    args_check = await check_arguments(username, drop_time, offset, bearer, combo, auth)

    if args_check is not None:
        logging.error(args_check)
        return None

    if not vps.count(":") == 2:
        logging.error("Invalid VPS format.")
    elif not vps.count("@") == 1:
        logging.error("Invalid VPS format.")

    client = Client()
    spl = vps.replace("@", ":").split(":")

    try:
        logging.info("Connecting to VPS...")
        client.connect(spl[2], int(spl[3]), spl[0], spl[1])
    except SSHException:
        logging.info("Error logging into VPS.")

    cmd = [f"wine geosnipe snipe {username} {drop_time} {offset}"]

    if bearer is not None:
        cmd.append(f"--bearer={bearer}")
    elif combo is not None:
        cmd.append(f"--combo={combo}")
        cmd.append(f"--auth={auth}")

    try:
        logging.info("Executing command...\n")
        stdin, stdout, stderr = client.exec_command(" ".join(cmd), get_pty=True)
    except SSHException:
        logging.info("Error executing command. Make sure GeoSnipe is installed on the VPS.")
        return None

    try:
        while True:
            line = stdout.readline()
            print(line[:len(line) - 2])

            if stdout.channel.exit_status_ready():
                line = stdout.readline()
                print(line[:len(line) - 2])
                break
            else:
                await asyncio.sleep(0.05)
    except SSHException:
        logging.info("Error executing command.")
        return None

    client.close()


async def sshoffset(vps: str, tests: int = 5) -> None:
    if tests < 1:
        logging.error("The argument `tests` cannot be less than or equal to 0.")
        return None

    if not vps.count(":") == 2:
        logging.error("Invalid VPS format.")
    elif not vps.count("@") == 1:
        logging.error("Invalid VPS format.")

    client = Client()
    spl = vps.replace("@", ":").split(":")

    try:
        logging.info("Connecting to VPS...")
        client.connect(spl[2], int(spl[3]), spl[0], spl[1])
    except SSHException:
        logging.info("Error logging into VPS.")

    try:
        logging.info("Executing command...\n")
        stdin, stdout, stderr = client.exec_command(f"wine geosnipe offset --tests={tests}", get_pty=True)
    except SSHException:
        logging.info("Error executing command. Make sure GeoSnipe is installed on the VPS.")
        return None

    try:
        while True:
            line = stdout.readline()
            print(line[:len(line) - 2])

            if stdout.channel.exit_status_ready():
                line = stdout.readline()
                print(line[:len(line) - 2])
                break
            else:
                await asyncio.sleep(0.05)
    except SSHException:
        logging.info("Error executing command.")
        return None

    client.close()


async def sshinstall(vps: str) -> None:
    if not vps.count(":") == 2:
        logging.error("Invalid VPS format.")
    elif not vps.count("@") == 1:
        logging.error("Invalid VPS format.")

    client = Client()
    spl = vps.replace("@", ":").split(":")

    logging.info("Finding latest download...")
    link = await GitHub.latest_download()

    if link is None:
        logging.error("Could not find latest download.")
        return

    logging.info("Found latest download.")

    try:
        logging.info("Connecting to VPS...")
        client.connect(spl[2], int(spl[3]), spl[0], spl[1])
    except SSHException:
        logging.info("Error logging into VPS.")

    commands = (
        "rm -f geosnipe.exe", "sudo dpkg --add-architecture i386",
        "sudo apt update", "sudo apt install -f wine64",
        "sudo apt install -f wine32", f"wget {link}"
    )

    for command in commands:
        try:
            logging.info(f"Executing command `{command}`...\n")
            stdin, stdout, stderr = client.exec_command(command, get_pty=True)
        except SSHException:
            logging.info("Error executing command. Make sure user has admin permissions.")
            return None

        try:
            while True:
                line = stdout.readline()
                print(line[:len(line) - 2])

                if stdout.channel.exit_status_ready():
                    line = stdout.readline()
                    print(line[:len(line) - 2])
                    break
                else:
                    await asyncio.sleep(0.05)
        except SSHException:
            logging.info("Error executing command.")
            return None



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import logging


__version__ = "v1.2.5"


class ColoredFormatter(logging.Formatter):
    def __init__(self):
        logging.Formatter.__init__(
            self, f"[\u001b[36mGeoSnipe\u001b[0m] [%(levelname)s] %(message)s"
        )

        self.colors = {
            "WARNING": "\u001b[33m",
            "INFO": "\u001b[34;1m",
            "CRITICAL": "\u001b[31;1m",
            "ERROR": "\u001b[31m"
        }

    def format(self, record):
        record.levelname = self.colors[record.levelname] + record.levelname + "\u001b[0m"
        return logging.Formatter.format(self, record)


logger = logging.getLogger("main")
handler = logging.StreamHandler()
handler.setFormatter(ColoredFormatter())
logger.addHandler(handler)
logger.setLevel(logging.INFO)



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

SKIN = b"""POST /minecraft/profile/skins HTTP/1.1\r
Host: api.minecraftservices.com\r
Authorization: Bearer %b\r
Content-Type: application/json\r
Content-Length: 99\r\n\r
{"variant": "classic", "url": "https://raw.githubusercontent.com/Geographs/GeoSnipe/main/skin.png"}"""

MOJANG = b"""PUT /minecraft/profile/name/%b HTTP/1.1\r
Host: api.minecraftservices.com\r
Authorization: Bearer %b\r
Content-Type: application/json\r\n\r\n"""

MICROSOFT = b"""POST /minecraft/profile HTTP/1.1\r
Host: api.minecraftservices.com\r
Authorization: Bearer %b\r
Content-Type: application/json
Content-Length: %b\r\n\r\n%b"""

PAYLOAD = b"""POST /authenticate HTTP/1.1\r
Host: authserver.mojang.com\r
Accept: application/json\r
Content-Type: application/json\r
Content-Length: %b\r
User-Agent: %b\r\n\r\n%b"""

USER_AGENT = b"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.164 " \
             b"Safari/537.36"



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from os import get_terminal_size


class ProgressBar(object):
    def __init__(self, total: int, symbols: tuple[str, str] = ("█", "░")) -> None:
        self.total = total
        self.symbols = symbols
        self.width = get_terminal_size().columns

        self.progress = 0
        print()
        self.print_progress_bar()

    def print_progress_bar(self):
        percentage = int(self.progress / self.total * 100)
        bar = self.symbols[0] * percentage + self.symbols[1] * (100 - percentage)
        msg = f"{percentage}% - [{bar}]"
        spaces = int((self.width - len(msg)) / 2)

        print(f"{' ' * spaces}{msg}", end="\r" if percentage != 100 else None)

        if percentage == 100:
            print()

    def update(self):
        if self.progress < self.total:
            self.progress += 1
            self.print_progress_bar()



import re
import asyncio


class GitHub(object):
    @staticmethod
    async def latest_download():
        try:
            reader, writer = await asyncio.open_connection("api.github.com", 443, ssl=True)
            writer.write(b"""GET /repos/Geographs/GeoSnipe/releases/latest HTTP/1.1\r\nHost: api.github.com\r
User-Agent: GeoSnipe\r\n\r\n""")
            await writer.drain()
            data = b""

            while True:
                data += await reader.read(1024)
                reader.feed_eof()
                if reader.at_eof():
                    break

            text = str(data, "utf-8")
            tag = re.findall("\"tag_name\":\"(.*?)\"", text)[0]
            return f"https://github.com/Geographs/GeoSnipe/releases/download/{tag}/geosnipe.exe"
        except IndexError:
            return None
        except ConnectionError:
            return None



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import sys
import asyncio

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))

from geosnipe import logger as logging
from geosnipe.argparser import parse_args
from geosnipe.functions import offset, snipe, sshoffset, sshsnipe, get_version, sshinstall


def initialize():
    os.system("")
    asyncio.set_event_loop(asyncio.ProactorEventLoop())


def main():
    initialize()

    loop = asyncio.get_event_loop()
    args = parse_args()

    if args.service_command is None:
        logging.error("No command selected. Use `geosnipe --help` for a list of commands.")
    elif args.service_command == "offset":
        loop.run_until_complete(offset(args.tests))
    elif args.service_command == "snipe":
        loop.run_until_complete(snipe(args.username, args.drop_time, args.offset, args.bearer, args.combo, args.auth))
    elif args.service_command == "sshoffset":
        loop.run_until_complete(sshoffset(args.vps, args.tests))
    elif args.service_command == "sshsnipe":
        loop.run_until_complete(
            sshsnipe(args.vps, args.username, args.drop_time, args.offset, args.bearer, args.combo, args.auth)
        )
    elif args.service_command == "sshinstall":
        loop.run_until_complete(sshinstall(args.vps))
    elif args.service_command == "version":
        loop.run_until_complete(get_version())


if __name__ == "__main__":
    main()



# MIT License
#
# Copyright (c) 2021 Geographs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import setuptools

long_description = open("README.md", "r", encoding="utf-8").read()

setuptools.setup(
    name="GeoSnipe",
    version="1.2.5",
    author="Geographs",
    maintainer="Geographs",
    author_email="87452561+Geographs@users.noreply.github.com",
    description="A pythonic Minecraft username sniper based on asyncio.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Geographs/GeoSnipe",
    download_url="https://github.com/Geographs/GeoSnipe/releases",
    project_urls={
        "GitHub": "https://github.com/Geographs/GeoSnipe",
        "Documentation": "https://docs.geographs.pro",
        "Bug Tracker": "https://github.com/Geographs/GeoSnipe/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=["geosnipe", "geosnipe.auth"],
    python_requires=">=3.9",
    license="MIT",
    entry_points={"console_scripts": ["geosnipe=geosnipe.__main__:main"]},
    install_requires=["msmcauth==0.0.3", "paramiko>=2.7.2"]
)
